# -*- coding: cp1252 -*-

# File:     TCAlgo_AlgoSteps.py
# Project: 	Implementation of sub-steps of the algorithm.
# Author:	Axel Newe (axel.newe@fau.de)
# Version:	1.0
# Date: 	2015-11-11

# ------------------------------------------------------------------------------------------------------

import scipy.optimize as opt
from TCAlgo_Tools import *
from TCAlgo_ReferenceFunctions import *

# ------------------------------------------------------------------------------------------------------

def CheckIfValuesAreInReferenceRange(labValuesEpisode):
    """
    This function checks if a series of lab values is within the referenceRange (i.e., [0..1])
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :return: True or False
    """

    result = False

    unbrokenReferenceDayArray, unbrokenLabValuesArray, sigmaArray = GetUnbrokenArraysFromLabValuesDict(labValuesEpisode, ALL_PHASES)

    if ((np.min(unbrokenLabValuesArray) >= 0) and (np.max(unbrokenLabValuesArray) <= 1)):

        result = True

    return result


def CheckForConstantCurve(labValuesEpisode, tolerance, consideredPhases):
    """
    This function checks if a series of lab values can be fit to a constant curve
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param tolerance: The (absolute) tolerance that is accepted for deviation from linearity
    :param consideredPhases: Specifies which phases (pre, mid, post) shall be considered.
    :return: True or False
    """

    result = False

    unbrokenReferenceDayArray, unbrokenLabValuesArray, sigma = GetUnbrokenArraysFromLabValuesDict(labValuesEpisode, consideredPhases)

    fitParams, fitCovariances = opt.curve_fit(ReferenceFunc_Constant, unbrokenReferenceDayArray, unbrokenLabValuesArray, None, sigma)
    fitted_c = fitParams[0]

    fittedCurve = ReferenceFunc_Constant(unbrokenReferenceDayArray, fitted_c)

    labValuesArrayWithoutGaps    = RemoveDummyValues(unbrokenLabValuesArray, sigma)
    fittedCurveWithoutGaps       = RemoveDummyValues(fittedCurve, sigma)

    delta_Values_to_FittedCurve        = np.array(labValuesArrayWithoutGaps - fittedCurveWithoutGaps)
    delta_Values_to_FittedCurve_StdDev = np.std(delta_Values_to_FittedCurve)

    if (delta_Values_to_FittedCurve_StdDev <= tolerance):
        result = True

    return result


def CheckForLinearCurve(labValuesEpisode, tolerance, consideredPhases):
    """
    This function checks if a series of lab value can be fit to a linear curve
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param tolerance: The (absolute) tolerance that is accepted for deviation from linearity
    :param consideredPhases: Specifies which phases (pre, mid, post) shall be considered.
    :return: True or False
    """

    result = False

    unbrokenReferenceDayArray, unbrokenLabValuesArray, sigma = GetUnbrokenArraysFromLabValuesDict(labValuesEpisode, consideredPhases)

    fitParams, fitCovariances = opt.curve_fit(ReferenceFunc_Linear, unbrokenReferenceDayArray, unbrokenLabValuesArray, None, sigma)
    fitted_a = fitParams[0]
    fitted_c = fitParams[1]

    fittedCurve = ReferenceFunc_Linear(unbrokenReferenceDayArray, fitted_a, fitted_c)

    labValuesArrayWithoutGaps = RemoveDummyValues(unbrokenLabValuesArray, sigma)
    fittedCurveWithoutGaps    = RemoveDummyValues(fittedCurve, sigma)

    delta_Values_to_FittedCurve        = np.array(labValuesArrayWithoutGaps - fittedCurveWithoutGaps)
    delta_Values_to_FittedCurve_StdDev = np.std(delta_Values_to_FittedCurve)

    if (delta_Values_to_FittedCurve_StdDev <= tolerance):
        result = True

    return result


def CheckIfMeanValueDiffersBetweenPhases(labValuesEpisode, tolerance):
    """
    This function checks if the mean values of the pre-phase, the mid-phase and the post phase differ significantly.
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param tolerance: The (absolute) tolerance that is accepted for deviation.
    :return: True or False
    """

    labValuesDifferInMean = False

    labValuesPrePhase, labValuesMedicationPhase, labValuesPostPhase = SplitLabValuesIntoPhases(labValuesEpisode)

    prePhaseMean        = np.mean(labValuesPrePhase)
    medicationPhaseMean = np.mean(labValuesMedicationPhase)
    postPhaseMean       = np.mean(labValuesPostPhase)

    labValueMedicationPlusPostPhase = np.concatenate((labValuesMedicationPhase,labValuesPostPhase))

    medicationPlusPostPhaseMean = np.mean(labValueMedicationPlusPostPhase)

    if (medicationPhaseMean > prePhaseMean + tolerance):
        labValuesDifferInMean = True

    if (postPhaseMean > prePhaseMean + tolerance):
        labValuesDifferInMean = True

    if (medicationPlusPostPhaseMean > prePhaseMean + tolerance):
        labValuesDifferInMean = True

    return labValuesDifferInMean


def CheckIfLabValuesAreFallingOrRising(labValuesEpisode, tolerance, consideredPhases):
    """
    This function checks if a series of lab value can be fit to a linear curve and if this curve is falling or rising.
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param tolerance: The (absolute) tolerance that is accepted for deviation from linearity
    :param consideredPhases: Specifies which phases (pre, mid, post) shall be considered.
    :return: True or False
    """

    result = 0

    unbrokenReferenceDayArray, unbrokenLabValuesArray, sigma = GetUnbrokenArraysFromLabValuesDict(labValuesEpisode, consideredPhases)

    fitParams, fitCovariances = opt.curve_fit(ReferenceFunc_Linear, unbrokenReferenceDayArray, unbrokenLabValuesArray, None, sigma)
    fitted_a = fitParams[0]

    if (abs(fitted_a) > tolerance):
        result = Signum(fitted_a)

    return result


def CheckIfLabValuesDifferFromMean(labValuesEpisode, tolerance):
    """
    This function checks if a series of lab values differs more than a given tolerance from its mean value.
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param tolerance: The (absolute) tolerance that is accepted for deviation from median
    :return: True or False
    """

    LabValuesDifferFromMean = False

    labValuesDict = labValuesEpisode[1]
    sortedLabValuesList = GetSortedLabValuesList(labValuesDict)

    labValuesMean = np.mean(sortedLabValuesList)

    labValuesDeltaToMean = np.abs(sortedLabValuesList - labValuesMean)

    labValuesDeltaToTolerance = labValuesDeltaToMean - tolerance

    maximumDeltaIndex = np.argmax(labValuesDeltaToMean)

    if (labValuesDeltaToTolerance[maximumDeltaIndex] > 0):
        LabValuesDifferFromMean = True

    return LabValuesDifferFromMean


def CheckIfLabValuesAlternateStrongly(labValuesEpisode, slopeTolerance, maxAllowedChanges):
    """
    This function checks if a series of lab values alternates strongly as regards the slope of each section.
    :param labValuesEpisode: A list containing the list of reference days, e.g. [0, 1, 2, 3, 4, 5] and a dictionary of lab values, e.g. {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}
    :param slopeTolerance: The (absolute) tolerance that is accepted for the slope.
    :param maxAllowedChanges: The number of slope changes that is accepted without yielding a True result.
    :return: True or False
    """

    LabValuesAlternateStrongly = False

    SlopesList = GetSlopesFromLabValuesDict(labValuesEpisode[1])
    NumSlopes = len(SlopesList)

    NumSlopeChangesMoreThanTolerance = 0

    for thisSlopeIndex in range(NumSlopes-2):
        thisSlope = SlopesList[thisSlopeIndex]
        nextSlope = SlopesList[thisSlopeIndex+1]
        nextNextSlope = SlopesList[thisSlopeIndex+2]

        if (abs(thisSlope) > slopeTolerance):
            if ( (abs(nextSlope) > slopeTolerance) and (Signum(thisSlope) != Signum(nextSlope))):
                if ( (abs(nextNextSlope) > slopeTolerance) and (Signum(nextSlope) != Signum(nextNextSlope))):
                    NumSlopeChangesMoreThanTolerance += 1

    if (NumSlopeChangesMoreThanTolerance > maxAllowedChanges):
        LabValuesAlternateStrongly = True

    return LabValuesAlternateStrongly







