# -*- coding: cp1252 -*-

# File:     TCAlgo_XMLTools.py
# Project: 	Tools for reading/writing XML files
# Author:	Axel Newe (axel.newe@fau.de)
# Version:	1.0
# Date: 	2015-11-11

# ------------------------------------------------------------------------------------------------------

import xml.dom.minidom as dom

# ------------------------------------------------------------------------------------------------------

###### XML Writing ##########################################################

def EncodeXMLTextBlock(textblock):
    """
    Encodes a string to UTF-8.
    :param textblock: The string to be encoded.
    :return: The encoded string.
    """

    result = ""

    result = textblock.encode('UTF-8')

    return result


def ExportCurveAssessmentDataToXML(fullPath, exportDataDict):
    """
    Writes the complete curve assessment data xml to the disk.
    :param fullPath: The full path name including file name.
    :param exportDataDict: The data dictionary that shall be exported. Example: {'000': ['temporal correlation', 0.87], '001': ['no change', 0.99], '002': ['no assessment', 0.55]}
    :return: Nothing.
    """

    print "Exporting result XML..."

    header = '<?xml version="1.0" encoding="UTF-8"?>\n'
    episode_data_open = '<curve_assessment_data>\n'
    episode_data_close = '</curve_assessment_data>\n'

    exportFile = open(fullPath,'w')
    exportFile.write(header)
    exportFile.write(episode_data_open)

    # Use indices to get rising numbers automatically
    for i in range(len(exportDataDict)):

        thisCurveNumber = str(i).rjust(3, '0')

        thisExportData = exportDataDict[thisCurveNumber]
        thisExportDataAssessment = thisExportData

        thisCurveBlock = '  <curve number="' + thisCurveNumber + '">' + thisExportDataAssessment + '</curve>\n'

        exportFile.write(thisCurveBlock)

    exportFile.write(episode_data_close)
    exportFile.close()

    print "Exporting result XML finished."

    return


###### XNL Reading ##########################################################


def GetMedicationDaysListFromCurveElement(curveElement):
    """
    Extracts the list of relative medication days from an XML element.
    :param curveElement: THe XML element to extract the data from.
    :return: A list of relative medication days.
    """

    medicationDaysList = []

    medicationNode = curveElement.getElementsByTagName("medication")[0]

    medicationValueNodes = medicationNode.getElementsByTagName("medication_value")

    for thisMedicationValueNode in medicationValueNodes:

        thismedicationReferenceDay = thisMedicationValueNode.getAttribute("reference_day_delta")

        medicationDaysList += [int(thismedicationReferenceDay)]

    return medicationDaysList


def GetNormalizedLabValuesDictFromCurveElement(curveElement):
    """
    Extracts the dictionary of lab values from an XML element.
    :param curveElement: The XML element to extract the data from.
    :return: A dictionary of lab values.
    """

    normalizedLabValuesDict = {}

    labValuesNode = curveElement.getElementsByTagName("lab_values")[0]

    labValueNodes = labValuesNode.getElementsByTagName("lab_value")

    for thisLabValueNode in labValueNodes:

        thisLabValueReferenceDay = int(thisLabValueNode.getAttribute("reference_day_delta"))
        thisLabValue             = float(thisLabValueNode.getAttribute("lab_value"))

        normalizedLabValuesDict[thisLabValueReferenceDay] = thisLabValue

    return normalizedLabValuesDict


def GetRootNodeFromXMLFile(fullPath):
    """
    Opens an XML file and returns the root node.
    :param fullPath: The full path including file name to the XML file.
    :return: The XML node of the root element.
    """

    XMLTree = dom.parse(fullPath)
    rootNode = XMLTree.documentElement

    return rootNode


def ImportDataXML(fullPath):
    """
    Imports the medication days and the lab values from the input data XML.
    :param fullPath: The full path including file name to the input data XML file.
    :return: A dictionary of medication days and lab values. Output format: {'000': [[0, 1, 2, 3, 4, 5, 6], {1: 0.25, 2: 0.167, 3: 0.111, 4: 0.222, 5: 0.361, 6: 0.333}]}
    """

    print "Importing Ground Truth XML..."

    normalizedLabValueEpisodesDict = {}

    rootNode = GetRootNodeFromXMLFile(fullPath)
    curveElements = rootNode.getElementsByTagName("curve")

    for thisCurveElement in curveElements:

        thisCurveNumber = thisCurveElement.getAttribute("number")

        medicationDaysList      = GetMedicationDaysListFromCurveElement(thisCurveElement)
        normalizedLabValuesDict = GetNormalizedLabValuesDictFromCurveElement(thisCurveElement)

        normalizedLabValueEpisodesDict[thisCurveNumber] = [medicationDaysList, normalizedLabValuesDict]

    print "Finished importing Ground Truth XML."

    return normalizedLabValueEpisodesDict
